@@ -9,7 +9,7 @@ Tk::ListEntry - BrowseEntry like widget without button
 use strict;
 use warnings;
 use vars qw($VERSION);
-$VERSION = '0.02';
+$VERSION = '0.04';
 
 use Tk;
 require Tk::PopList;
@@ -37,9 +37,15 @@ You can use all config options and methods of the Entry widget.
 
 Callback to be executed when you press the B<Return> button or select an item in the list.
 
+=item Switch: B<-filter>
+
+By default set to false. If you set it you can use the entry widget to filter the list.
+
 =item Switch: B<-motionselect>
 
-See L<Tk::PopList>.
+Default value 1
+
+Selects list item when set hoovering over it.
 
 =item Switch: B<-popdirection>
 
@@ -71,21 +77,31 @@ sub Populate {
 
 	my $entry = $self->Entry->pack(-expand => 1, -fill => 'both');
 	my $list = $self->PopList(
+		-nofocus => 1,
 		-confine => 1,
 		-selectcall => ['EntrySelect', $self],
 		-widget => $entry,
 	);
+	$list->bind('<Button-1>',[$self, 'ListClick', Ev('y')]);
 	$self->Advertise(Entry => $entry);
 	$self->Advertise(List => $list);
+	$list->Subwidget('List')->bind('<Button-1>', [$list, 'Select']);
 	$entry->bind('<Button-1>', [$list, 'popFlip']);
-	$entry->bind('<Down>', [$list, 'popUp']);
-	$entry->bind('<FocusOut>', [$self, 'EntryFocusOut', Ev('d')]);
-	$entry->bind('<Return>', [$self, 'EntrySelect', Ev('d')]);
+	$entry->bind('<Down>', [$self, 'keyDown']);
+	$entry->bind('<End>', [$self, 'keyEnd']);
+	$entry->bind('<Home>', [$self, 'keyHome']);
+	$entry->bind('<Escape>', [$list, 'popDown']);
+	$entry->bind('<FocusOut>', [$list, 'popDown']);
+	$entry->bind('<KeyRelease>', [$self, 'filter']);
+	$entry->bind('<Return>', [$self, 'keyReturn']);
+	$entry->bind('<Up>', [$self, 'keyUp']);
 
 
 	$self->ConfigSpecs(
 		-background => ['SELF', 'DESCENDANTS'],
 		-command => ['CALLBACK', undef, undef, sub {}],
+		-filter => ['PASSIVE', undef, undef, 0],
+		-foreground => [$entry],
 		-motionselect => [$list],
 		-popdirection => [$list],
 		-values => [$list],
@@ -102,20 +118,6 @@ sub Populate {
 
 =cut
 
-sub EntryFocusOut {
-	my ($self, $detail) = @_;
-	my $pl = $self->Subwidget('List');
-	my $l = $pl->Subwidget('Listbox')->Subwidget('listbox');
-	my $fc = $self->focusCurrent;
-	unless (defined $fc) {
-		$pl->popDown;
-	} else {
-		unless ($fc->focusCurrent eq $l) {
-			$pl->popDown;
-		} 
-	}
-}
-
 sub EntrySelect {
 	my ($self, $select) = @_;
 	my $entry = $self->Subwidget('Entry');
@@ -126,6 +128,89 @@ sub EntrySelect {
 	$self->Callback('-command', $entry->get);
 }
 
+sub filter {
+	my $self = shift;
+	return unless $self->cget('-filter');
+	my $l = $self->Subwidget('List');
+	my $e = $self->Subwidget('Entry');
+	$l->filter($e->get);
+}
+
+sub keyDown {
+	my $self = shift;
+	my $l = $self->Subwidget('List');
+	my $hl = $l->Subwidget('List');
+	my ($sel) = $hl->selectionGet;
+	if ($l->ismapped) {
+		unless (defined $sel) {
+			$hl->selectionSet(0) unless defined $sel;
+			$hl->anchorSet(0) unless defined $sel;
+		} else {
+			$l->NavDown
+		}
+	} else {
+		$hl->anchorClear;
+		$l->popUp
+	}
+}
+
+sub keyEnd {
+	my $self = shift;
+	my $l = $self->Subwidget('List');
+	my $e = $self->Subwidget('Entry');
+	if ($l->ismapped) {
+		$l->NavLast
+	} else {
+		$e->icursor('end');
+	}
+}
+
+sub keyHome {
+	my $self = shift;
+	my $l = $self->Subwidget('List');
+	my $e = $self->Subwidget('Entry');
+	if ($l->ismapped) {
+		$l->NavFirst
+	} else {
+		$e->icursor(0);
+	}
+}
+
+sub keyReturn {
+	my $self = shift;
+	my $l = $self->Subwidget('List');
+	my @sel = $l->Subwidget('List')->selectionGet;
+	if (($l->ismapped) and (@sel)) {
+		$l->Select
+	} else {
+		my $e = $self->Subwidget('Entry');
+		$self->Callback('-command', $e->get);
+	}
+
+}
+
+sub keyUp {;
+	my $self = shift;
+	my $l = $self->Subwidget('List');
+	if ($l->ismapped) {
+		$l->NavUp
+	} 
+}
+
+#This method is a hack to make the ListEntry
+#also work with -motionselect disabled
+
+sub ListClick {
+	my ($self, $y) = @_;
+	my $list = $self->Subwidget('List')->Subwidget('List');
+	my $near = $list->nearest($y);
+	if ((defined $near) and (! $self->cget('-motionselect'))) {
+		$list->selectionSet($near);
+		my $call = $list->cget('-browsecmd');
+		$call->Call;
+	}
+}
+
 =item B<validate>
 
 
@@ -167,6 +252,4 @@ Unknown. If you find any, please contact the author.
 
 =cut
 
-1;
-
-
+1;
\ No newline at end of file
